<html>
<body>

<h1>
  Jeton d'authentification invalide. </h1>

<p>
  Nous sommes désolés, mais votre jeton d'authentification est invalide ou
  expirés. </p>

<p> Vous pouvez vous désabonner des notifications en vous <a href="/login/">
  connectant puis en allant dans le menu "Préférences" en haut à droite de
  l'écran.</a>
</p>

</body>
</html>
