// Wiki

.wiki-page {
  &-info {
    border-top: 1px solid #eee;
    padding-top: 3px;
    color: #aaa;
  }

  &-body {
    margin: 10px 0;
    font-size: 14px;
  }
}

.wikilink.new {
  color: #ba0000;

  &:hover {
    color: #ba0000;
  }
}
